@@ -1,10 +1,10 @@
+use breezyshim::branch::PyBranch;
 use breezyshim::debian::directory::vcs_git_url_to_bzr_url;
-use breezyshim::tree::Tree;
-use breezyshim::workingtree::WorkingTree;
+use breezyshim::tree::{PyTree, Tree};
+use breezyshim::workingtree::{GenericWorkingTree, WorkingTree};
 use clap::Parser;
 use debversion::Version;
 use log::warn;
-use pyo3::prelude::*;
 use std::collections::HashMap;
 
 use std::io::Write as _;
@@ -21,6 +21,35 @@ use ognibuild::debian::fix_build::IterateBuildError;
 use ognibuild::dependencies::debian::DebianDependency;
 use upstream_ontologist::{Certainty, UpstreamDatum, UpstreamDatumWithMetadata, UpstreamMetadata};
 
+#[derive(Debug, Clone)]
+enum SessionType {
+    Plain,
+    Schroot(Option<String>),
+    Unshare(Option<PathBuf>),
+}
+
+impl std::str::FromStr for SessionType {
+    type Err = String;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        if let Some((session_type, value)) = s.split_once(':') {
+            match session_type {
+                "plain" => Ok(SessionType::Plain),
+                "schroot" => Ok(SessionType::Schroot(Some(value.to_string()))),
+                "unshare" => Ok(SessionType::Unshare(Some(PathBuf::from(value)))),
+                _ => Err(format!("Invalid session type: {}", session_type)),
+            }
+        } else {
+            match s {
+                "plain" => Ok(SessionType::Plain),
+                "schroot" => Ok(SessionType::Schroot(None)),
+                "unshare" => Ok(SessionType::Unshare(None)),
+                _ => Err(format!("Invalid session type: {}", s)),
+            }
+        }
+    }
+}
+
 /// Create Debian packaging for upstream projects, in version control
 #[derive(Parser, Debug)]
 #[command(author, version)]
@@ -77,13 +106,9 @@ struct Args {
     #[arg(long, short('i'))]
     install: bool,
 
-    /// Schroot to use for building apt archive access
-    #[arg(long, env = "SCHROOT")]
-    schroot: Option<String>,
-
-    /// Unshare tarball to use for building apt archive access
-    #[arg(long)]
-    unshare: Option<PathBuf>,
+    /// Session type for isolation: plain, schroot[:name], unshare[:tarball]
+    #[arg(long, default_value = "unshare")]
+    session: SessionType,
 
     /// Build command (used for --iterate-fix)
     #[arg(long, default_value_t = format!("{} -A -s v", debian_analyzer::DEFAULT_BUILDER))]
@@ -142,6 +167,14 @@ struct Args {
 
     /// Upstream to package
     upstream: Option<String>,
+
+    /// Package requirement specification (e.g., ">=1.0.0")
+    #[arg(long)]
+    requirement: Option<String>,
+
+    /// Force specific build system (override detection)
+    #[arg(long)]
+    buildsystem: Option<String>,
 }
 
 fn main() -> Result<(), i32> {
@@ -189,7 +222,7 @@ fn main() -> Result<(), i32> {
     let create_dist_fn: Option<
         Box<
             dyn for<'a, 'b, 'c, 'd, 'e> Fn(
-                &'a dyn Tree,
+                &'a dyn PyTree,
                 &'b str,
                 &'c Version,
                 &'d Path,
@@ -199,7 +232,7 @@ fn main() -> Result<(), i32> {
         >,
     > = if let Some(dist_command) = dist_command {
         Some(Box::new(
-            move |tree: &dyn Tree,
+            move |tree: &dyn PyTree,
                   package: &str,
                   version: &Version,
                   target_dir: &Path,
@@ -226,7 +259,7 @@ fn main() -> Result<(), i32> {
 
     // For now...
     let (upstream_branch, upstream_subpath) = if let Some(upstream) = args.upstream {
-        match breezyshim::branch::open_containing(&upstream.parse().unwrap()) {
+        match breezyshim::branch::open_containing_as_generic(&upstream.parse().unwrap()) {
             Ok((upstream_branch, upstream_subpath)) => {
                 metadata.insert(UpstreamDatumWithMetadata {
                     datum: UpstreamDatum::Repository(upstream),
@@ -257,7 +290,7 @@ fn main() -> Result<(), i32> {
         if let Some(url) = upstream_info.repository() {
             log::info!("Found relevant upstream branch at {}", url);
             let (upstream_branch, upstream_subpath) =
-                breezyshim::branch::open_containing(&url.parse().unwrap()).unwrap();
+                breezyshim::branch::open_containing_as_generic(&url.parse().unwrap()).unwrap();
 
             metadata.insert(UpstreamDatumWithMetadata {
                 datum: UpstreamDatum::Repository(url.to_owned()),
@@ -320,14 +353,30 @@ fn main() -> Result<(), i32> {
         minimum_certainty: debian_analyzer::Certainty::Confident,
         consult_external_directory: args.consult_external_directory,
         verbose: args.verbose,
-        session: if let Some(schroot) = args.schroot.as_ref() {
-            log::info!("Using schroot {}", schroot);
-            SessionPreferences::Schroot(schroot.to_string())
-        } else if let Some(unshare) = args.unshare.as_ref() {
-            log::info!("Using tarball {} for unshare", unshare.display());
-            SessionPreferences::Unshare(unshare.to_path_buf())
-        } else {
-            SessionPreferences::Plain
+        session: match args.session {
+            SessionType::Plain => {
+                log::info!("Using plain session (no isolation)");
+                SessionPreferences::Plain
+            }
+            SessionType::Schroot(ref name) => {
+                let schroot_name = name.clone().or_else(|| std::env::var("SCHROOT").ok());
+                if let Some(name) = schroot_name {
+                    log::info!("Using schroot: {}", name);
+                    SessionPreferences::Schroot(name)
+                } else {
+                    log::info!("Using default schroot session");
+                    SessionPreferences::Schroot("unstable".to_string()) // Default schroot name
+                }
+            }
+            SessionType::Unshare(ref tarball) => {
+                if let Some(path) = tarball {
+                    log::info!("Using unshare with tarball: {}", path.display());
+                    SessionPreferences::Unshare(path.clone())
+                } else {
+                    log::info!("Using default unshare session");
+                    SessionPreferences::Unshare(PathBuf::new()) // Empty path for default
+                }
+            }
         },
         create_dist: create_dist_fn,
         committer: None,
@@ -335,6 +384,9 @@ fn main() -> Result<(), i32> {
         debian_revision: args.debian_revision,
         team: None,
         author: None,
+        compat_level: None,
+        check_wnpp: true,
+        run_fixers: true,
     };
 
     let lock_write = wt.lock_write();
@@ -342,7 +394,7 @@ fn main() -> Result<(), i32> {
     let debianize_result = match debianize::debianize(
         &wt,
         &subpath,
-        Some(upstream_branch.as_ref()),
+        Some(&upstream_branch as &dyn PyBranch),
         Some(&upstream_subpath),
         &preferences,
         args.upstream_version.as_deref(),
@@ -455,6 +507,38 @@ fn main() -> Result<(), i32> {
                 None,
             );
         }
+        Err(Error::SourceNameUnknown(name)) => {
+            svp.report_fatal(
+                "source-name-unknown",
+                &format!(
+                    "Unable to determine source package name{}",
+                    name.as_ref()
+                        .map(|n| format!(" from {}", n))
+                        .unwrap_or_default()
+                ),
+                None,
+                None,
+            );
+        }
+        Err(Error::Other(msg)) => {
+            svp.report_fatal("other-error", &msg, None, None);
+        }
+        Err(Error::ProviderError(e)) => {
+            svp.report_fatal(
+                "provider-error",
+                &format!("Error getting upstream metadata: {:?}", e),
+                None,
+                None,
+            );
+        }
+        Err(Error::UncommittedChanges) => {
+            svp.report_fatal(
+                "uncommitted-changes",
+                "There are uncommitted changes in the working tree.",
+                Some("Commit your changes or use --force to ignore them."),
+                None,
+            );
+        }
     };
 
     std::mem::drop(lock_write);
@@ -465,35 +549,57 @@ fn main() -> Result<(), i32> {
 
     if args.iterate_fix {
         #[cfg(target_os = "linux")]
-        let session: std::rc::Rc<dyn ognibuild::session::Session> =
-            if let Some(schroot) = args.schroot.as_ref() {
-                log::info!("Using schroot {}", schroot);
-                std::rc::Rc::new(
-                    ognibuild::session::schroot::SchrootSession::new(schroot, None).unwrap(),
-                ) as _
-            } else if let Some(unshare) = args.unshare.as_ref() {
-                log::info!("Using tarball {} for unshare", unshare.display());
-                std::rc::Rc::new(
-                    ognibuild::session::unshare::UnshareSession::from_tarball(unshare).unwrap(),
-                ) as _
-            } else {
+        let session: std::rc::Rc<dyn ognibuild::session::Session> = match &args.session {
+            SessionType::Plain => {
                 std::rc::Rc::new(ognibuild::session::plain::PlainSession::new()) as _
-            };
+            }
+            SessionType::Schroot(ref name) => {
+                let schroot_name = name
+                    .clone()
+                    .or_else(|| std::env::var("SCHROOT").ok())
+                    .unwrap_or_else(|| "unstable".to_string());
+                log::info!("Using schroot {}", schroot_name);
+                std::rc::Rc::new(
+                    ognibuild::session::schroot::SchrootSession::new(&schroot_name, None).unwrap(),
+                ) as _
+            }
+            SessionType::Unshare(ref tarball) => {
+                if let Some(path) = tarball {
+                    log::info!("Using tarball {} for unshare", path.display());
+                    std::rc::Rc::new(
+                        ognibuild::session::unshare::UnshareSession::from_tarball(path).unwrap(),
+                    ) as _
+                } else {
+                    log::info!("Using bootstrapped unshare session");
+                    std::rc::Rc::new(
+                        ognibuild::session::unshare::UnshareSession::bootstrap().unwrap(),
+                    ) as _
+                }
+            }
+        };
 
         #[cfg(not(target_os = "linux"))]
         let session = std::rc::Rc::new(ognibuild::session::plain::PlainSession::new());
 
-        let mut tempdir = None;
+        let mut _tempdir = None;
 
         let output_directory = if args.discard_output {
-            tempdir = Some(tempfile::tempdir().unwrap());
-            tempdir.as_ref().unwrap().path().to_path_buf()
+            _tempdir = Some(tempfile::tempdir().unwrap());
+            _tempdir.as_ref().unwrap().path().to_path_buf()
         } else if let Some(output_directory) = args.output_directory {
             output_directory
         } else {
-            let output_directory = debianize::default_debianize_cache_dir().unwrap();
-            log::info!("Building dependencies in {}", output_directory.display());
-            output_directory
+            match debianize::default_debianize_cache_dir() {
+                Ok(dir) => {
+                    log::info!("Building dependencies in {}", dir.display());
+                    dir
+                }
+                Err(e) => {
+                    log::warn!("Failed to access XDG cache directory: {}. Using temporary directory instead.", e);
+                    _tempdir = Some(tempfile::tempdir().unwrap());
+                    _tempdir.as_ref().unwrap().path().to_path_buf()
+                }
+            }
         };
 
         let committer = preferences
@@ -503,7 +609,7 @@ fn main() -> Result<(), i32> {
             .clone();
         let build_command = args.build_command.clone();
 
-        let do_build = move |wt: &WorkingTree,
+        let do_build = move |wt: &GenericWorkingTree,
                              subpath: &Path,
                              incoming_directory: &Path,
                              extra_repositories: Vec<&str>|
@@ -513,7 +619,7 @@ fn main() -> Result<(), i32> {
         > {
             let apt = ognibuild::debian::apt::AptManager::from_session(session.as_ref());
             let context = ognibuild::debian::context::DebianPackagingContext::new(
-                wt.clone(),
+                Clone::clone(wt),
                 subpath,
                 committer.clone(),
                 false,
@@ -672,30 +778,5 @@ fn versions_dict() -> HashMap<String, String> {
         "lintian-brush".to_string(),
         env!("CARGO_PKG_VERSION").to_string(),
     );
-    pyo3::Python::with_gil(|py| {
-        let breezy = py.import_bound("breezy").unwrap();
-        ret.insert(
-            "breezy".to_string(),
-            breezy.getattr("version_string").unwrap().extract().unwrap(),
-        );
-
-        let debmutate = py.import_bound("debmutate").unwrap();
-        ret.insert(
-            "debmutate".to_string(),
-            debmutate
-                .getattr("version_string")
-                .unwrap()
-                .extract()
-                .unwrap(),
-        );
-
-        let debian = py.import_bound("debian").unwrap();
-        ret.insert(
-            "debian".to_string(),
-            debian.getattr("__version__").unwrap().extract().unwrap(),
-        );
-
-        // TODO(jelmer): Read dependencies from Cargo.lock
-    });
     ret
 }
